// implementation for LSM303DLHC Accelerometer/compass
// This applies to the STM32L476 Discovery board only
// Accelerometer,Magnetometer and gyro are on SPI2 bus
// PD4 = MOSI
// PD3 = MISO
// PD1 = SCK
// PD7 = Gyro CS
// PE0 = Accelerometer CS
// PC0 = Magnetometer CS
#include <stdint.h>
#include "lsm303dlhc.h"
#include "stm32l476.h"
#include "spi2.h"
void initAccel()
{
	WriteAccelRegister(CTRL_REG4_A,1);  // enable 3 wire read and write
                                            // disable I2C interface
        WriteAccelRegister(CTRL_REG1_A,0x77); // power up the accelerometer

}
void initCompass()
{
        WriteCompassRegister(CTRL_REG1_M, 0xfc); // Temp sensor on, max data rate for x and y, max output data rate
        WriteCompassRegister(CTRL_REG2_M, 0x60); // +/- 16 gauss
	WriteCompassRegister(CTRL_REG3_M, 0x84); // Disable I2C interface and enable SPI read/write
        WriteCompassRegister(CTRL_REG4_M, 0x0c); // select ultra high performance mode for z axis
}

void WriteAccelRegister(uint8_t SubAddress, uint8_t Data)
{
	// Assert CS for accelerometer
	GPIOE_ODR &= ~BIT0; 	
	SubAddress = SubAddress & ~BIT7; // ensure MSB = 0 for write mode
	spi_write(SubAddress,Data);
	// set accelerometer into spi idle mode
	GPIOE_ODR |= BIT0; 	

}
uint8_t ReadAccelRegister(uint8_t SubAddress)
{
	uint8_t RValue;
	// Assert CS for accelerometer
	GPIOE_ODR &= ~BIT0; 	
	SubAddress=SubAddress | 0x80;
	RValue = spi_read3Wire(SubAddress);
	// set accelerometer into spi idle mode
	GPIOE_ODR |= BIT0; 	
	return RValue;
}

void WriteCompassRegister(uint8_t SubAddress, uint8_t Value)
{
	
	// Assert CS for compass
	GPIOC_ODR &= ~BIT0; 
	SubAddress = SubAddress & ~BIT8; // ensure MSB = 0 for write mode
	spi_write(SubAddress,Value);
	// Release CS for compass
	GPIOC_ODR |= BIT0; 
}
uint8_t ReadCompassRegister(uint8_t SubAddress )
{
	uint8_t RValue;
	// Assert CS for compass
	GPIOC_ODR &= ~BIT0; 
	SubAddress = SubAddress | 0x80; // ensure MSB = 1 for read mode
	RValue = spi_read3Wire(SubAddress);
	// Release CS for compass
	GPIOC_ODR |= BIT0; 
	return RValue;
}

