#include "spi2.h"
// This applies to the STM32L476 Discovery board only
// Accelerometer,Magnetometer and gyro are on SPI2 bus
// PD4 = MOSI
// PD3 = MISO
// PD1 = SCK
// PD7 = Gyro CS
// PE0 = Accelerometer CS
// PC0 = Magnetometer CS
void delay(unsigned dly); // this function is in main.c
void initSPI2()
{
	// Turn on SPI2 clock
	RCC_APB1ENR1 |= BIT14;
	// reset the SPI2 interface
	RCC_APB1RSTR1 |= BIT14;
	delay(10);
	RCC_APB1RSTR1 &= ~BIT14;
	// Accelerometer/Compass & Gyro use on Ports C,D and E
	// Turn on GPIOC, GPIOD and GPIOE
	RCC_AHB2ENR |= (BIT2 + BIT3 + BIT4);
	// Select alternate function for PD1,PD3,PD4
	GPIOD_MODER |= (BIT3+BIT7+BIT9);
	GPIOD_MODER &= ~(BIT2+BIT6+BIT8);
	// Alternate function 5 = SPI2 on PORTD pins (see datasheet page 76)
	GPIOD_AFRL &= 0xfff00f0f; // clear upper bits
	GPIOD_AFRL |= 0x00055050; // select AF5 for bits 1, 3 and 4

	// PC0 is a general purpose output
	GPIOC_MODER &= ~(BIT1);
	GPIOC_MODER |= BIT0;
	// PD7 is a general purpose output
	GPIOD_MODER &= ~(BIT15);
	GPIOD_MODER |= BIT14;
	// PE0 is a general purpose output port
	GPIOE_MODER &= ~(BIT1);
	GPIOE_MODER |= (BIT0);
		
	
	// set magnetometer into spi idle mode
	GPIOC_ODR |= BIT0; 	
	// set gyro into spi idle mode
	GPIOD_ODR |= BIT7; 	
	// set accelerometer into spi idle mode
	GPIOE_ODR |= BIT0; 	
	
	SPI2_CR2 = (0x07 << 8); // 8 bits per transfer, 1 byte threshold	
	// setup the SPI operating modes etc
	/*
		Bidirectional mode (BIT15 = 1 for bidirectional mode)
		Software slave management (chip select controlled by softare)
		Internal slave select 
		8 databits (Bit 7 = 0)
		Enable SPI
		Freq = fpclk/256
		Master
		CPOL (CK = 1 when idle)
		CPHA (Data valid on second clock edge)
	*/
	SPI2_CR1 = BIT9+BIT8+BIT6+(7<<3)+BIT2+BIT1+BIT0;
}
void spi_write(int SubAddress, int Value)
{
	short RValue;
	int timeout;
	timeout=SPI_TIMEOUT;
	//GPIOE_ODR &= ~BIT3; // Drive CS low
	SPI2_DR = (short)((Value << 8)+SubAddress);
	RValue=SPI2_SR;
	while ( (timeout--) && ((SPI2_SR & BIT7)!=0) ); // wait for tx complete
	RValue = SPI2_DR; // dummy read of data register
	//GPIOE_ODR |= BIT3;  // Drive CS High
}
int spi_read4Wire(int SubAddress)
{
	// 4-wire SPI read: MOSI,MISO,SCK and CS
	short RValue;
	int timeout;
	timeout=SPI_TIMEOUT;
	SPI2_CR1 |= BIT14; // transmit mode
	SPI2_DR = ((SubAddress | 0x80)); // output subaddress + read flag
	while ((timeout--) && ((SPI2_SR & BIT7)!=0) ); // wait for tx complete
	RValue = SPI2_DR >>8 ;
	return RValue;
}
int spi_read3Wire(int SubAddress)
{
	// 3-wire SPI read: MOSI,SCK and CS.  MOSI line is bidirectional
	short RValue=0;
	int LongRValue = 0;
	int timeout;
	SPI2_CR1 |= BIT14;
	SPI2_CR1 |= BIT15+BIT14; // bidirectional mode, transmit mode
	SubAddress = SubAddress + (SubAddress << 8); // not clear which byte gets sent to LSM so do both
	SPI2_DR = (short)(SubAddress); // output subaddress + read flag	
	timeout=SPI_TIMEOUT;
	while ((timeout--) && ((SPI2_SR & BIT7)!=0) ); // wait for tx complete	
	SPI2_CR2 |= BIT12;  // do 8 bit reads
	SPI2_CR1 &= ~BIT14; // begin receive mode		
	timeout = SPI_TIMEOUT;
	while((timeout--)&&(SPI2_SR & BIT0)==0);
	SPI2_CR1 |= BIT14; // end receive mode	
	RValue = SPI2_DR;
	LongRValue = RValue;
	SPI2_CR2 &= ~BIT12;  // undo 8 bit reads
	SPI2_CR1 |= BIT14; // end receive mode
	SPI2_CR1 &= ~BIT15; // turn off Bidirectional mode
	return LongRValue;
}
void ClearSPIMODF()
{
	int dummy = SPI2_SR; 
	dummy = SPI2_CR1;
	SPI2_CR1 = dummy;
}
// Functions below here are used for development/testing only
int spi_read4WireWWorking(int SubAddress)
{
	// 4-wire SPI read: MOSI,MISO,SCK and CS
	short RValue;
	int timeout;
	timeout=SPI_TIMEOUT;
	SPI2_CR1 |= BIT14; // transmit mode
	//GPIOE_ODR &= ~BIT3; // Drive CS low
	SPI2_DR = ((SubAddress | 0x80)); // output subaddress + read flag
	while ((timeout--) && ((SPI2_SR & BIT7)!=0) ); // wait for tx complete
	RValue = SPI2_DR >>8 ;
	//GPIOE_ODR |= BIT3;  // Drive CS High
	return RValue;
}
int DebugArray[10];
int spi_read3WireWorking(int SubAddress)
{
	// 3-wire SPI read: MOSI,SCK and CS.  MOSI line is bidirectional
	short RValue=0;
	int LongRValue = 0;
	int timeout;
	SPI2_CR1 |= BIT15+BIT14; // bidirectional mode, transmit mode
	SubAddress = SubAddress + (SubAddress << 8); // not clear which byte gets sent to LSM so do both
	DebugArray[0]=SPI2_SR;
	SPI2_DR = (short)(SubAddress); // output subaddress + read flag	
	DebugArray[1]=SPI2_SR;
	timeout=SPI_TIMEOUT;
	while ((timeout--) && ((SPI2_SR & BIT7)!=0) ); // wait for tx complete
	
	DebugArray[2]=SPI2_SR;
	SPI2_CR2 |= BIT12;  // do 8 bit reads
	SPI2_CR1 &= ~BIT14; // begin receive mode		
	DebugArray[3]=SPI2_SR;
	timeout = SPI_TIMEOUT;
	while((timeout--)&&(SPI2_SR & BIT0)==0);
	//delay(1000);
	DebugArray[4]=SPI2_SR;
	SPI2_CR1 |= BIT14; // end receive mode	
	RValue = SPI2_DR;
	DebugArray[5]=SPI2_SR;
	LongRValue = RValue;
	SPI2_CR2 &= ~BIT12;  // undo 8 bit reads
	SPI2_CR1 |= BIT14; // end receive mode
	SPI1_CR2 &= ~BIT15; // turn off Bidirectional mode
	return LongRValue;
	
}
